/*
 Chapter 10 - A real case
 Greenhouse control
 By Francis Perea for Packt Publishing
*/

// Libraries inclusion
#include <Servo.h>

// Constans used in the code
// USed pins
#define redLedPin 13
#define yellowLedPin 12
#define greenLedPin 11
#define relayPin 8
#define motorPin 6
#define buzzerPin 4
#define servoPin 3
#define buttonPin 2
#define potentiometerPin 2
#define thermistorPin 1
#define photocellPin 0
// Notes for the alarm
#define NOTE_A4 440
#define NOTE_A3 220
// Range of valid temperatures
#define TEMPMIN 820
#define TEMPMAX 850
// Power for the fan
#define MEDIUMPOWER 128
#define FULLPOWER 255
// Watering range
#define BEGINWATERING 700
#define ENDWATERING 550
// Light cutoff
#define DAYLIGHT 400

// Global variables
// A boolean flag for the ISR
volatile boolean buttonPressed = false;
// Values of the sensors
int tempValue;
int humidityValue;
int lightValue;
// A servo instance
Servo myServo;
// A mapped state
int state;

// The ISR of the interrupt 0
void buttonPress(){
  // Set the flag up
  buttonPressed = true;
}

// Actions to be taken in case of emergency
void alarmRoutine(){
  // Close the roof
  myServo.write(0);
  // Turn on water pump
  digitalWrite(relayPin,HIGH);
  // Generate sound
  tone(buzzerPin,NOTE_A4);
  // All LEDs on
  digitalWrite(greenLedPin,HIGH);
  digitalWrite(yellowLedPin,HIGH);
  digitalWrite(redLedPin,HIGH);
  delay(1000);
  // Generate a different sound
  tone(buzzerPin,NOTE_A3);
  // All LEDs off
  digitalWrite(greenLedPin,LOW);
  digitalWrite(yellowLedPin,LOW);
  digitalWrite(redLedPin,LOW);
  delay(1000);
  // Turn sound off
  noTone(buzzerPin);
}

// Board configuration and initialization
void setup() {
  // Configure pins
  pinMode(redLedPin, OUTPUT);
  pinMode(yellowLedPin, OUTPUT);
  pinMode(greenLedPin, OUTPUT);
  pinMode(relayPin,OUTPUT);
  pinMode(motorPin, OUTPUT); 
  pinMode(buzzerPin, OUTPUT);
  // Initialize servo
  myServo.attach(servoPin);
  // Set the interrupt ISR
  attachInterrupt(0, buttonPress, RISING);
  // Begin serial feedback
  Serial.begin(9600);
}


void loop(){  
  // Read the temperature
  tempValue = analogRead(thermistorPin);
  // map it to a 4 states range
  state =map(tempValue, TEMPMIN, TEMPMAX, 1, 4);
  switch (state){
    // All ok
    case 1: 
     // Simply green LED on.
     digitalWrite(greenLedPin,HIGH);
     break;
    // Temperature raising
    case 2:
      // Yellow LED ON
      digitalWrite(yellowLedPin,HIGH);
      // Fan at medium speed
      analogWrite(motorPin, MEDIUMPOWER);
      break;
    // Getting hot
    case 3:
      // Red LED ON
      digitalWrite(redLedPin, HIGH);
      // Fan at full throttle
      analogWrite(motorPin,FULLPOWER);
      // Open the retractable roof
      myServo.write(180);
      break;
    // Things had gone worst
    case 4:
      // Triger the alarm routine
      alarmRoutine();
      break;
  }
  
  // Read the humidity sensor
  humidityValue=analogRead(potentiometerPin);
  // if too dry
  if (humidityValue>BEGINWATERING){
    // Begin watering
    digitalWrite(relayPin,HIGH);
  }
  // If too humid
  if (humidityValue<ENDWATERING){
    // Stop watering
    digitalWrite(relayPin,LOW);
  }
  
  // Read the light sensor
  lightValue = analogRead(photocellPin);
  // If it is night
  if (lightValue>DAYLIGHT){
    // Close the retractable roof
    myServo.write(0);
  }
  // Otherwise open it
  else{
    myServo.write(180);
  }
  
  // If a manual alarm was triggered
  if (buttonPressed){
    // Rest it
    buttonPressed = false;
    // Execute alarm routine
    alarmRoutine();  
  }
  
  // Send data back
  Serial.print(" S: ");
  Serial.print(state);
  Serial.print(" T: ");
  Serial.print(tempValue);
  Serial.print(" H: ");
  Serial.print(humidityValue);
  Serial.print(" L: ");
  Serial.println(lightValue);
}

