/*
 Chapter 09 - Dealing with interrupt
 Complete tachograph system
 By Francis Perea for Packt Publishing
*/

#include <Servo.h>

//The pins that will be used
#define ledPin 13
#define motorPin 6
#define buzzerPin 4
#define servoPin 3

#define NOTE_A4 440
// Milliseconds between every sample
#define sampleTime 500
// Motor speed increment
#define motorIncrement 10
// Range of valir RPMs, alarm and stop
#define minRPM  0
#define maxRPM 10000
#define alarmRPM 8000
#define stopRPM 9000

// Global variables we will use
// A variable to be used inside ISR
volatile unsigned long revolutions = 0;
// Total number of revolutions in every sample
long lastSampleRevolutions = 0;
// A variable to convert revolutions per sample to RPM
int rpm = 0;
// LED Status
int ledStatus = LOW;
// An instace on the Servo class
Servo myServo;
// A flag to know if the motor has been stalled
boolean motorStalled = false;
// Thr current dial angle
int dialAngle = 0;
// A variable to store serial data
int dataReceived;
// The current motor speed
int speed = 0;
// A time variable to compare in every sample
unsigned long lastCheckTime;

// A function to be called when the interrupt occurs
void revolution(){
  // Increment the total number of 
  // revolutions in the current sample
  revolutions++;
}

// Configuration of the board
void setup() {
  // Set output pins
  pinMode(motorPin, OUTPUT);
  pinMode(ledPin, OUTPUT); 
  pinMode(buzzerPin, OUTPUT);
  // Set revolution() as ISR of interrupt 0
  attachInterrupt(0, revolution, CHANGE);
  // Init serial communication
  Serial.begin(9600);
  // Initialize the servo
  myServo.attach(servoPin);
  //Set the dial
  myServo.write(dialAngle);
  // Initialize the counter for sample time
  lastCheckTime = millis();
}

// Sketch execution loop
void loop(){  
  // If we have received serial data
  if (Serial.available()) {
    // read the next char
     dataReceived = Serial.read();
     // Act depending on it
     switch (dataReceived){
       // Increment speed
       case '+':
         if (speed<250) {
           speed += motorIncrement;
         }
         break;
       // Decrement speed
       case '-':
         if (speed>5) {
           speed -= motorIncrement;
         }
         break;         
       // Stop motor
       case '0':
         speed = 0;
         break;     
       // Full throttle    
       case '*':
         speed = 255;
         break;
       // Reactivate motor after stall
       case 'R':
         speed = 0;
         motorStalled = false;
         break;
     }
    //Only if motor is active set new motor speed 
    if (motorStalled == false){
      // Set the speed motor speed
      analogWrite(motorPin, speed);
    }
  }
  // If a sample time has passed
  // We have to take another sample
  if (millis() - lastCheckTime > sampleTime){
    // Store current revolutions 
    lastSampleRevolutions = revolutions;
    // Reset the global variable
    // So the ISR can begin to count again
    revolutions = 0;
    // Calculate revolution per minute
    rpm = lastSampleRevolutions * (1000 / sampleTime) * 60;
    // Update last sample time
    lastCheckTime = millis();
    // Set the dial according new reading
    dialAngle = map(rpm,minRPM,maxRPM,180,0);
    myServo.write(dialAngle);
  }
  // If the motor is running in the red zone
  if (rpm > alarmRPM){
    // Turn on LED
    digitalWrite(ledPin, HIGH);
  }
  else{
    // Otherwise turn it off
    digitalWrite(ledPin, LOW);
  }
  // If the motor has exceed maximum RPM
  if (rpm > stopRPM){
    // Stop the motor
    speed = 0;
    analogWrite(motorPin, speed);
    // Disable it until a ‘R’ command is received
    motorStalled = true;
    // Make alarm sound
    tone(buzzerPin, NOTE_A4, 1000);
  }
  // Send data back to the computer
  Serial.print("RPM: ");
  Serial.print(rpm);
  Serial.print(" SPEED: ");
  Serial.print(speed);
  Serial.print(" STALL: ");
  Serial.println(motorStalled);
}

